@@ -1,4 +1,5 @@
 """Widget generators and their signal handlers"""
+
 # Standard Library
 # pylint: disable=no-member,too-many-public-methods
 import os
@@ -9,11 +10,14 @@ from gi.repository import Gdk, Gtk
 
 # Lutris Modules
 from lutris import settings, sysoptions
+from lutris.config import LutrisConfig
+from lutris.game import Game
 from lutris.gui.widgets.common import EditableGrid, FileChooserEntry, Label, VBox
 from lutris.gui.widgets.searchable_combobox import SearchableCombobox
 from lutris.runners import InvalidRunnerError, import_runner
 from lutris.util.log import logger
 from lutris.util.strings import gtk_safe
+from lutris.util.wine.wine import clear_wine_version_cache
 
 
 class ConfigBox(VBox):
@@ -21,10 +25,11 @@ class ConfigBox(VBox):
 
     config_section = NotImplemented
 
-    def __init__(self, config_level, game=None):
+    def __init__(self, config_level: str, lutris_config: LutrisConfig, game: Game = None) -> None:
         super().__init__()
         self.options = []
         self.config_level = config_level
+        self.lutris_config = lutris_config
         self.game = game
         self.config = None
         self.raw_config = None
@@ -148,6 +153,13 @@ class ConfigBox(VBox):
                 option_key = option["option"]
                 value = self.config.get(option_key)
 
+                if "visible" in option:
+                    if callable(option["visible"]):
+                        option["visible"] = option["visible"]()
+
+                    if not option["visible"]:
+                        continue
+
                 if callable(option.get("choices")) and option["type"] != "choice_with_search":
                     option["choices"] = option["choices"]()
                 if callable(option.get("condition")):
@@ -208,8 +220,7 @@ class ConfigBox(VBox):
                 if value != default and option_key not in self.raw_config:
                     helptext = helptext + "\n\n" if helptext else ""
                     helptext += _(
-                        "<i>(Italic indicates that this option is "
-                        "modified in a lower configuration level.)</i>"
+                        "<i>(Italic indicates that this option is " "modified in a lower configuration level.)</i>"
                     )
                 if helptext:
                     self.wrapper.props.has_tooltip = True
@@ -219,18 +230,18 @@ class ConfigBox(VBox):
                 option_container = hbox
                 hbox.set_margin_left(18)
                 hbox.pack_end(placeholder, False, False, 5)
+                hbox.pack_start(self.wrapper, True, True, 0)
+
                 # Grey out option if condition unmet
                 if "condition" in option and not option["condition"]:
-                    hbox.set_sensitive(False)
-
-                hbox.pack_start(self.wrapper, True, True, 0)
+                    self.wrapper.set_sensitive(False)
 
                 if "warning" in option:
                     option_body = option_container
                     option_container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, visible=True)
                     option_container.pack_start(option_body, False, False, 0)
                     warning = ConfigWarningBox(option["warning"], option_key)
-                    warning.update_warning(self.config)
+                    warning.update_warning(self.lutris_config)
                     self.warning_boxes[option_key] = warning
                     option_container.pack_start(warning, False, False, 0)
 
@@ -238,8 +249,8 @@ class ConfigBox(VBox):
                     option_body = option_container
                     option_container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, visible=True)
                     option_container.pack_start(option_body, False, False, 0)
-                    error = ConfigErrorBox(option["error"], option_key, hbox)
-                    error.update_warning(self.config)
+                    error = ConfigErrorBox(option["error"], option_key, self.wrapper)
+                    error.update_warning(self.lutris_config)
                     self.error_boxes[option_key] = error
                     option_container.pack_start(error, False, False, 0)
 
@@ -258,12 +269,12 @@ class ConfigBox(VBox):
         show_advanced = settings.read_setting("show_advanced_options") == "True"
         self.advanced_visibility = show_advanced
 
-    def update_warnings(self):
+    def update_warnings(self) -> None:
         for box in self.warning_boxes.values():
-            box.update_warning(self.config)
+            box.update_warning(self.lutris_config)
 
         for box in self.error_boxes.values():
-            box.update_warning(self.config)
+            box.update_warning(self.lutris_config)
 
     def call_widget_generator(self, option, option_key, value, default):  # noqa: C901
         """Call the right generation method depending on option type."""
@@ -379,8 +390,8 @@ class ConfigBox(VBox):
     def on_searchable_entry_changed(self, combobox, value, key):
         self.option_changed(combobox, key, value)
 
-    def _populate_combobox_choices(self, liststore, choices, default):
-        expanded, tooltip_default = self._expand_combobox_choices(choices, default)
+    def _populate_combobox_choices(self, liststore, choices, value, default):
+        expanded, tooltip_default = self._expand_combobox_choices(choices, value, default)
         for choice in expanded:
             liststore.append(choice)
 
@@ -388,23 +399,28 @@ class ConfigBox(VBox):
             self.tooltip_default = tooltip_default
 
     @staticmethod
-    def _expand_combobox_choices(choices, default):
+    def _expand_combobox_choices(choices, value, default):
         expanded = []
         tooltip_default = None
+        has_value = False
         for choice in choices:
             if isinstance(choice, str):
                 choice = (choice, choice)
+            if choice[1] == value:
+                has_value = True
             if choice[1] == default:
                 tooltip_default = choice[0]
                 choice = (_("%s (default)") % choice[0], choice[1])
             expanded.append(choice)
+        if not has_value and value:
+            expanded.insert(0, (value + " (invalid)", value))
         return expanded, tooltip_default
 
     # ComboBox
     def generate_combobox(self, option_name, choices, label, value=None, default=None, has_entry=False):
         """Generate a combobox (drop-down menu)."""
         liststore = Gtk.ListStore(str, str)
-        self._populate_combobox_choices(liststore, choices, default)
+        self._populate_combobox_choices(liststore, choices, value, default)
         # With entry ("choice_with_entry" type)
         if has_entry:
             combobox = Gtk.ComboBox.new_with_model_and_entry(liststore)
@@ -418,7 +434,7 @@ class ConfigBox(VBox):
 
         combobox.set_id_column(1)
 
-        expanded, _tooltip_default = self._expand_combobox_choices(choices, default)
+        expanded, _tooltip_default = self._expand_combobox_choices(choices, value, default)
         if value in [v for _k, v in expanded]:
             combobox.set_active_id(value)
         elif has_entry:
@@ -495,9 +511,8 @@ class ConfigBox(VBox):
             warn_if_non_writable_parent=warn_if_non_writable_parent,
             text=path,
             default_path=chooser_default_path,
-            shell_quoting=shell_quoting
+            shell_quoting=shell_quoting,
         )
-        # file_chooser.set_size_request(200, 30)
 
         if "default_path" in option:
             chooser_default_path = self.lutris_config.system_config.get(option["default_path"])
@@ -538,7 +553,7 @@ class ConfigBox(VBox):
             action=Gtk.FileChooserAction.SELECT_FOLDER,
             warn_if_non_writable_parent=warn_if_non_writable_parent,
             text=path,
-            default_path=chooser_default_path
+            default_path=chooser_default_path,
         )
         directory_chooser.connect("changed", self._on_chooser_file_set, option_name)
         directory_chooser.set_valign(Gtk.Align.CENTER)
@@ -664,10 +679,10 @@ class ConfigBox(VBox):
         label = Label(text)
         label.set_use_markup(True)
         label.set_max_width_chars(60)
-        hbox = Gtk.Box()
-        hbox.pack_start(label, False, False, 0)
-        hbox.show_all()
-        tooltip.set_custom(hbox)
+        event_box = Gtk.EventBox()
+        event_box.add(label)
+        event_box.show_all()
+        tooltip.set_custom(event_box)
         return True
 
     def option_changed(self, widget, option_name, value):
@@ -692,7 +707,7 @@ class ConfigBox(VBox):
 
         btn.set_visible(False)
         self.set_style_property("font-weight", "normal", wrapper)
-        self.raw_config.pop(option_key)
+        self.raw_config.pop(option_key, None)
         self.lutris_config.update_cascaded_config()
 
         reset_value = self.config.get(option_key)
@@ -737,9 +752,8 @@ class ConfigBox(VBox):
 class GameBox(ConfigBox):
     config_section = "game"
 
-    def __init__(self, config_level, lutris_config, game):
-        ConfigBox.__init__(self, config_level, game)
-        self.lutris_config = lutris_config
+    def __init__(self, config_level: str, lutris_config: LutrisConfig, game: Game):
+        ConfigBox.__init__(self, config_level, lutris_config, game)
         self.runner = game.runner
         if self.runner:
             self.options = self.runner.game_options
@@ -752,9 +766,9 @@ class RunnerBox(ConfigBox):
 
     config_section = "runner"
 
-    def __init__(self, config_level, lutris_config, game=None):
-        ConfigBox.__init__(self, config_level, game)
-        self.lutris_config = lutris_config
+    def __init__(self, config_level: str, lutris_config: LutrisConfig, game: Game = None):
+        ConfigBox.__init__(self, config_level, lutris_config, game)
+
         try:
             self.runner = import_runner(self.lutris_config.runner_slug)()
         except InvalidRunnerError:
@@ -763,18 +777,22 @@ class RunnerBox(ConfigBox):
             self.options = self.runner.get_runner_options()
 
         if lutris_config.level == "game":
-            self.generate_top_info_box(_(
-                "If modified, these options supersede the same options from "
-                "the base runner configuration."
-            ))
+            self.generate_top_info_box(
+                _("If modified, these options supersede the same options from " "the base runner configuration.")
+            )
+
+    def generate_widgets(self):
+        # Better safe than sorry - we search of Wine versions in directories
+        # we do not control, so let's keep up to date more aggresively.
+        clear_wine_version_cache()
+        return super().generate_widgets()
 
 
 class SystemConfigBox(ConfigBox):
     config_section = "system"
 
-    def __init__(self, config_level, lutris_config):
-        ConfigBox.__init__(self, config_level)
-        self.lutris_config = lutris_config
+    def __init__(self, config_level: str, lutris_config: LutrisConfig) -> None:
+        ConfigBox.__init__(self, config_level, lutris_config)
         self.runner = None
         runner_slug = self.lutris_config.runner_slug
 
@@ -784,24 +802,31 @@ class SystemConfigBox(ConfigBox):
             self.options = sysoptions.system_options
 
         if lutris_config.game_config_id and runner_slug:
-            self.generate_top_info_box(_(
-                "If modified, these options supersede the same options from "
-                "the base runner configuration, which themselves supersede "
-                "the global preferences."
-            ))
+            self.generate_top_info_box(
+                _(
+                    "If modified, these options supersede the same options from "
+                    "the base runner configuration, which themselves supersede "
+                    "the global preferences."
+                )
+            )
         elif runner_slug:
-            self.generate_top_info_box(_(
-                "If modified, these options supersede the same options from "
-                "the global preferences."
-            ))
+            self.generate_top_info_box(
+                _("If modified, these options supersede the same options from " "the global preferences.")
+            )
 
 
 class UnderslungMessageBox(Gtk.Box):
     """A box to display a message with an icon inside the configuration dialog."""
 
     def __init__(self, icon_name, margin_left=18, margin_right=18, margin_bottom=6):
-        super().__init__(spacing=6, visible=False, margin_left=margin_left, margin_right=margin_right,
-                         margin_bottom=margin_bottom, no_show_all=True)
+        super().__init__(
+            spacing=6,
+            visible=False,
+            margin_left=margin_left,
+            margin_right=margin_right,
+            margin_bottom=margin_bottom,
+            no_show_all=True,
+        )
 
         image = Gtk.Image(visible=True)
         image.set_from_icon_name(icon_name, Gtk.IconSize.DND)
@@ -834,7 +859,7 @@ class ConfigMessageBox(UnderslungMessageBox):
             if text:
                 self.label.set_markup(str(text))
 
-    def update_warning(self, config):
+    def update_warning(self, config: LutrisConfig) -> None:
         try:
             if callable(self.warning):
                 text = self.warning(config, self.option_key)
@@ -842,7 +867,7 @@ class ConfigMessageBox(UnderslungMessageBox):
                 text = self.warning
         except Exception as err:
             logger.exception("Unable to generate configuration warning: %s", err)
-            text = gtk_safe(err)
+            text = gtk_safe(str(err))
 
         return self.show_markup(text)
 
@@ -853,11 +878,11 @@ class ConfigWarningBox(ConfigMessageBox):
 
 
 class ConfigErrorBox(ConfigMessageBox):
-    def __init__(self, error, option_key, option_container):
+    def __init__(self, error, option_key, wrapper):
         super().__init__(error, option_key, icon_name="dialog-error")
-        self.option_container = option_container
+        self.wrapper = wrapper
 
-    def update_warning(self, config):
+    def update_warning(self, config: LutrisConfig) -> None:
         visible = super().update_warning(config)
-        self.option_container.set_sensitive(not visible)
+        self.wrapper.set_sensitive(not visible)
         return visible
